/* =======================================
	   DARK MODE STYLES
======================================= */

/* Dark Mode Toggle Button */
.litespeed-dark-mode-toggle {
	position: fixed;
	top: 32px;
	right: 20px;
	z-index: 999999;
	background: none;
	border: none;
	width: auto;
	height: auto;
	color: inherit;
	cursor: pointer;
	font-size: 16px;
}

/* =======================================
   		  DARK MODE STYLES
   Auto-applied based on browser preference
   OR manually toggled with .litespeed-darkmode class
======================================= */

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) {
		background-color: #1e1e1e;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode {
	background-color: #1e1e1e;
	color: #e0e0e0;
}

/* WordPress admin wrapper */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) #wpwrap {
		background-color: #1e1e1e;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode #wpwrap {
	background-color: #1e1e1e;
	color: #e0e0e0;
}

/* Main content area */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) #wpcontent {
		background-color: #181818;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode #wpcontent {
	background-color: #181818;
	color: #e0e0e0;
}

/* Form inputs */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='text'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='number'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='email'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='url'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='password'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='search'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='file'],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) textarea,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) select {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode input[type='text'],
body.litespeed-darkmode input[type='number'],
body.litespeed-darkmode input[type='email'],
body.litespeed-darkmode input[type='url'],
body.litespeed-darkmode input[type='password'],
body.litespeed-darkmode input[type='search'],
body.litespeed-darkmode input[type='file'],
body.litespeed-darkmode textarea,
body.litespeed-darkmode select {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='text']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='number']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='email']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='url']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='password']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='search']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type='file']:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) textarea:focus,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) select:focus {
		background-color: #404043;
		border: 1px solid #5a5a5d;
		box-shadow: 0 0 0 1px #0073aa;
	}
}
body.litespeed-darkmode input[type='text']:focus,
body.litespeed-darkmode input[type='number']:focus,
body.litespeed-darkmode input[type='email']:focus,
body.litespeed-darkmode input[type='url']:focus,
body.litespeed-darkmode input[type='password']:focus,
body.litespeed-darkmode input[type='search']:focus,
body.litespeed-darkmode input[type='file']:focus,
body.litespeed-darkmode textarea:focus,
body.litespeed-darkmode select:focus {
	background-color: #404043;
	border: 1px solid #5a5a5d;
	box-shadow: 0 0 0 1px #0073aa;
}

/* Buttons */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-secondary {
		background-color: #3e3e42;
		border-color: #5a5a5d;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .button,
body.litespeed-darkmode .button-secondary {
	background-color: #3e3e42;
	border-color: #5a5a5d;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) circle.litespeed-pie_bg {
		stroke: #3e3e42;
	}
}
body.litespeed-darkmode circle.litespeed-pie_bg {
	stroke: #3e3e42;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) g.litespeed-pie_info text {
		fill: #e0e0e0;
	}
}
body.litespeed-darkmode g.litespeed-pie_info text {
	fill: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) g.litespeed-pie_info .litespeed-pie-done {
		fill: #6dd17e;
	}
}
body.litespeed-darkmode g.litespeed-pie_info .litespeed-pie-done {
	fill: #6dd17e;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-switch input:not(:checked) + label {
		background-color: #3e3e42;
		color: #e0e0e0;
		border: 1px solid #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-switch input:not(:checked) + label {
	background-color: #3e3e42;
	color: #e0e0e0;
	border: 1px solid #5a5a5d;
}

/* Column with boxes layout */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-column-with-boxes .postbox {
		border-color: #3e3e42;
	}
}
body.litespeed-darkmode .litespeed-column-with-boxes .postbox {
	border-color: #3e3e42;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-column-with-boxes > div.litespeed-column-right {
		background-color: #2d2d30;
	}
}
body.litespeed-darkmode .litespeed-column-with-boxes > div.litespeed-column-right {
	background-color: #2d2d30;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-image-optim-summary,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-column-left-inside {
		background-color: #1e1e1e;
		border: 1px solid #3e3e42;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-image-optim-summary,
body.litespeed-darkmode .litespeed-column-left-inside {
	background-color: #1e1e1e;
	border: 1px solid #3e3e42;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed"]:not(.litespeed-lightmode).litespeed-cache_page_litespeed-img_optm [data-litespeed-layout='summary'],
	body[class*="litespeed"]:not(.litespeed-lightmode).litespeed-cache_page_litespeed-img_optm .litespeed-column-with-boxes > div.litespeed-column-right,
	body[class*="litespeed"]:not(.litespeed-lightmode).litespeed-cache_page_litespeed-cdn .litespeed-column-with-boxes > div.litespeed-column-right,
	body[class*="litespeed"]:not(.litespeed-lightmode).litespeed-cache_page_litespeed-cdn [data-litespeed-layout='qc'] {
		background-color: #181818;
	}
}
body.litespeed-darkmode.litespeed-cache_page_litespeed-img_optm [data-litespeed-layout='summary'],
body.litespeed-darkmode.litespeed-cache_page_litespeed-img_optm .litespeed-column-with-boxes > div.litespeed-column-right,
body.litespeed-darkmode.litespeed-cache_page_litespeed-cdn .litespeed-column-with-boxes > div.litespeed-column-right,
body.litespeed-darkmode.litespeed-cache_page_litespeed-cdn [data-litespeed-layout='qc'] {
	background-color: #181818;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-image-optim-summary-footer,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-column-with-boxes-footer {
		border-top: 1px solid #373737;
		background: revert;
	}
}
body.litespeed-darkmode .litespeed-image-optim-summary-footer,
body.litespeed-darkmode .litespeed-column-with-boxes-footer {
	border-top: 1px solid #373737;
	background: revert;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-column-with-boxes-footer {
		border-top: 1px solid #3e3e42;
		background-color: #353539;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-column-with-boxes-footer{
	border-top: 1px solid #3e3e42;
	background-color: #353539;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .nav-tab:focus:not(.nav-tab-active),
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .nav-tab:hover:not(.nav-tab-active) {
		background-color: #4d4d51;
		border-color: #666;
		color: #fff
	}
}
body.litespeed-darkmode .nav-tab:focus:not(.nav-tab-active),
body.litespeed-darkmode .nav-tab:hover:not(.nav-tab-active) {
	background-color: #4d4d51;
	border-color: #666;
	color: #fff
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button:hover,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-secondary:hover {
		background-color: #484851;
		border-color: #666;
	}
}
body.litespeed-darkmode .button:hover,
body.litespeed-darkmode .button-secondary:hover {
	background-color: #484851;
	border-color: #666;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-primary {
		background-color: #0073aa;
		border-color: #005a87;
		color: #ffffff;
	}
}
body.litespeed-darkmode .button-primary {
	background-color: #0073aa;
	border-color: #005a87;
	color: #ffffff;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-primary:hover {
		background-color: #005a87;
		border-color: #004a73;
	}
}
body.litespeed-darkmode .button-primary:hover {
	background-color: #005a87;
	border-color: #004a73;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-primary:disabled,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .button-primary[disabled] {
		background-color: #3e3e42;
		border-color: #5a5a5d;
		color: #8c8f94;
		cursor: not-allowed;
		opacity: 0.6;
	}
}
body.litespeed-darkmode .button-primary:disabled,
body.litespeed-darkmode .button-primary[disabled] {
	background-color: #3e3e42;
	border-color: #5a5a5d;
	color: #8c8f94;
	cursor: not-allowed;
	opacity: 0.6;
}

/* Danger buttons */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-btn-danger-bg,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed .litespeed-btn-danger-bg,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-wrap .litespeed-btn-danger-bg {
		background-color: #dc3232;
		border-color: #b32d2e;
		color: #ffffff;
		box-shadow: 0 1px 0 rgba(179, 45, 46, 0.5);
	}
}
body.litespeed-darkmode .litespeed-btn-danger-bg,
body.litespeed-darkmode .litespeed .litespeed-btn-danger-bg,
body.litespeed-darkmode .litespeed-wrap .litespeed-btn-danger-bg {
	background-color: #dc3232;
	border-color: #b32d2e;
	color: #ffffff;
	box-shadow: 0 1px 0 rgba(179, 45, 46, 0.5);
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-btn-danger-bg:hover,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed .litespeed-btn-danger-bg:hover,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-wrap .litespeed-btn-danger-bg:hover {
		background-color: #b32d2e;
		border-color: #a02622;
	}
}
body.litespeed-darkmode .litespeed-btn-danger-bg:hover,
body.litespeed-darkmode .litespeed .litespeed-btn-danger-bg:hover,
body.litespeed-darkmode .litespeed-wrap .litespeed-btn-danger-bg:hover {
	background-color: #b32d2e;
	border-color: #a02622;
}

/* Notices */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .notice {
		background-color: #262626;
		border-left: 4px solid #72a9d3;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .notice {
	background-color: #262626;
	border-left: 4px solid #72a9d3;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .notice-success {
		border-left-color: #46b450;
	}
}
body.litespeed-darkmode .notice-success {
	border-left-color: #46b450;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .notice-warning {
		border-left-color: #b28100;
	}
}
body.litespeed-darkmode .notice-warning {
	border-left-color: #b28100;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .notice-error {
		border-left-color: #dc3232;
	}
}
body.litespeed-darkmode .notice-error {
	border-left-color: #dc3232;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .notice-info {
		border-left-color: #00a0d2;
	}
}
body.litespeed-darkmode .notice-info {
	border-left-color: #00a0d2;
}

/* Striped table notices */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .striped > tbody > :nth-child(even) .notice {
		background-color: #3e3e42;
		box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.3);
	}
}
body.litespeed-darkmode .striped > tbody > :nth-child(even) .notice {
	background-color: #3e3e42;
	box-shadow: 0 1px 1px 0 rgba(0, 0, 0, 0.3);
}

/* Postboxes */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .postbox {
		background-color: #2d2d30;
		border: 1px solid #767679;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .postbox {
	background-color: #2d2d30;
	border: 1px solid #767679;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .postbox h3,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .postbox .hndle {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .postbox h3,
body.litespeed-darkmode .postbox .hndle {
	color: #e0e0e0;
}

/* Tables */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) table,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .wp-list-table {
		background-color: #2d2d30;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode table,
body.litespeed-darkmode .wp-list-table {
	background-color: #2d2d30;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .wp-list-table th {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .wp-list-table th {
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .wp-list-table td {
		border-bottom: 1px solid #3e3e42;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .wp-list-table td {
	border-bottom: 1px solid #3e3e42;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .striped > tbody > :nth-child(odd) {
		background-color: #2d2d30;
	}
}
body.litespeed-darkmode .striped > tbody > :nth-child(odd) {
	background-color: #2d2d30;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .striped > tbody > :nth-child(even) {
		background-color: #353539;
	}
}
body.litespeed-darkmode .striped > tbody > :nth-child(even) {
	background-color: #353539;
}

/* Form tables */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .form-table {
		background: transparent;
	}
}
body.litespeed-darkmode .form-table {
	background: transparent;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .form-table th {
		background: transparent;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .form-table th {
	background: transparent;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .form-table td {
		background: transparent;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .form-table td {
	background: transparent;
	color: #e0e0e0;
}

/* Links */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) a {
		color: #72aee6;
	}
}
body.litespeed-darkmode a {
	color: #72aee6;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) a:hover,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) a:focus {
		color: #9ec8f2;
	}
}
body.litespeed-darkmode a:hover,
body.litespeed-darkmode a:focus {
	color: #9ec8f2;
}

/* Code blocks */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) code {
		background-color: #404043;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode code {
	background-color: #404043;
	color: #e0e0e0;
}

/* Horizontal rules */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) hr {
		border-color: #5a5a5d;
		background-color: #5a5a5d;
	}
}
body.litespeed-darkmode hr {
	border-color: #5a5a5d;
	background-color: #5a5a5d;
}

/* Dashboard widgets */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .dashboard-widget {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
	}
}
body.litespeed-darkmode .dashboard-widget {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .dashboard-widget .widget-top {
		background-color: #404043;
		border-bottom: 1px solid #5a5a5d;
	}
}
body.litespeed-darkmode .dashboard-widget .widget-top {
	background-color: #404043;
	border-bottom: 1px solid #5a5a5d;
}

/* Meta boxes */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .meta-box-sortables .postbox {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
	}
}
body.litespeed-darkmode .meta-box-sortables .postbox {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
}

/* LiteSpeed specific styles */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-header {
		border-bottom: 1px solid #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-header {
	border-bottom: 1px solid #5a5a5d;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-body {
		background-color: #1e1e1e;
		border-left: 1px solid #5a5a5d;
		border-right: 1px solid #5a5a5d;
		border-bottom: 1px solid #5a5a5d;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-body {
	background-color: #1e1e1e;
	border-left: 1px solid #5a5a5d;
	border-right: 1px solid #5a5a5d;
	border-bottom: 1px solid #5a5a5d;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-panel {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-panel {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-panel-wrapper {
		background: transparent;
	}
}
body.litespeed-darkmode .litespeed-panel-wrapper {
	background: transparent;
}

/* Dashboard titles */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-h1,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) h1.litespeed-h1 {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-h1,
body.litespeed-darkmode h1.litespeed-h1 {
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) h3 {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode h3 {
	color: #e0e0e0;
}

/* LiteSpeed postboxes */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-postbox {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox .inside {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-postbox .inside {
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox .inside .litespeed-title {
		border-bottom: 1px solid #5a5a5d;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-postbox .inside .litespeed-title {
	border-bottom: 1px solid #5a5a5d;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox .inside.litespeed-postbox-footer {
		background-color: #353539;
		border-top: 1px solid #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-postbox .inside.litespeed-postbox-footer {
	background-color: #353539;
	border-top: 1px solid #5a5a5d;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox p.litespeed-dashboard-stats-total {
		border-top: 1px dashed #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-postbox p.litespeed-dashboard-stats-total {
	border-top: 1px dashed #5a5a5d;
}

/* Dashboard stats */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-stats .litespeed-desc {
		color: #b3b3b3;
	}
}
body.litespeed-darkmode .litespeed-dashboard-stats .litespeed-desc {
	color: #b3b3b3;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-stats-payg {
		color: #b3b3b3;
	}
}
body.litespeed-darkmode .litespeed-dashboard-stats-payg {
	color: #b3b3b3;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-stats-payg strong {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-dashboard-stats-payg strong {
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-stats-wrapper .litespeed-postbox:not(:first-child) {
		border-left-color: #3e3e42;
	}
}
body.litespeed-darkmode .litespeed-dashboard-stats-wrapper .litespeed-postbox:not(:first-child) {
	border-left-color: #3e3e42;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox.litespeed-postbox-partner h3.litespeed-title {
		color: #b3b3b3;
	}
}
body.litespeed-darkmode .litespeed-postbox.litespeed-postbox-partner h3.litespeed-title {
	color: #b3b3b3;
}

/* QUIC.cloud postbox styling */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox--quiccloud {
		border-color: #3e7cb3;
	}
}
body.litespeed-darkmode .litespeed-postbox--quiccloud {
	border-color: #3e7cb3;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title {
		background-color: #41464a;
		color: #ffffff;
	}
}
body.litespeed-darkmode .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title {
	background-color: #41464a;
	color: #ffffff;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title a {
		color: #a5caf2;
	}
}
body.litespeed-darkmode .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title a {
	color: #a5caf2;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title a:hover {
		color: #c2dcff;
	}
}
body.litespeed-darkmode .litespeed-postbox--quiccloud.litespeed-postbox .inside .litespeed-title a:hover {
	color: #c2dcff;
}

/* Dashboard unlock/promo styling */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-unlock {
		background-color: #2d2d30;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-dashboard-unlock {
	background-color: #2d2d30;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-unlock-desc {
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-dashboard-unlock-desc {
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-dashboard-unlock-desc span {
		color: #b3b3b3;
	}
}
body.litespeed-darkmode .litespeed-dashboard-unlock-desc span {
	color: #b3b3b3;
}

/* Navigation tabs */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .nav-tab {
		background-color: #3e3e42;
		border-color: #5a5a5d;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .nav-tab {
	background-color: #3e3e42;
	border-color: #5a5a5d;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .nav-tab:hover {
		background-color: #1e1e1e;
	}
}
body.litespeed-darkmode .nav-tab:hover {
	background-color: #1e1e1e;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .nav-tab-active {
		background-color: #1e1e1e;
		border-bottom-color: #1e1e1e;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .nav-tab-active {
	background-color: #1e1e1e;
	border-bottom-color: #1e1e1e;
	color: #e0e0e0;
}

/* Status indicators */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-success {
		color: #46b450;
	}
}
body.litespeed-darkmode .litespeed-success {
	color: #46b450;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-error {
		color: #dc3232;
	}
}
body.litespeed-darkmode .litespeed-error {
	color: #dc3232;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-warning {
		color: #ffb900;
	}
}
body.litespeed-darkmode .litespeed-warning {
	color: #ffb900;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-warning-bg {
		background-color: #584007 !important;
		color: #ffffff;
	}
}
body.litespeed-darkmode .litespeed-warning-bg {
	background-color: #584007 !important;
	color: #ffffff;
}

/* Footer */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) #wpfooter {
		background-color: #2d2d30;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode #wpfooter {
	background-color: #2d2d30;
	color: #e0e0e0;
}

/* Checkbox and radio inputs */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type="checkbox"],
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) input[type="radio"] {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
	}
}
body.litespeed-darkmode input[type="checkbox"],
body.litespeed-darkmode input[type="radio"] {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
}

/* Progress bars */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-progress {
		background-color: #3e3e42;
	}
}
body.litespeed-darkmode .litespeed-progress {
	background-color: #3e3e42;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-progress-bar {
		background-color: #0073aa;
	}
}
body.litespeed-darkmode .litespeed-progress-bar {
	background-color: #0073aa;
}

/* Litespeed tick buttons */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick {
		background-color: #2d2d30;
		border: 1px solid #3e3e42;
		padding: 8px 10px;
	}
}
body.litespeed-darkmode .litespeed-tick {
	background-color: #2d2d30;
	border: 1px solid #3e3e42;
	padding: 8px 10px;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick:hover {
		background-color: #404043;
		border-color: #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-tick:hover {
	background-color: #404043;
	border-color: #5a5a5d;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick label {
		color: #e0e0e0;
		padding: 4px 8px;
	}
}
body.litespeed-darkmode .litespeed-tick label {
	color: #e0e0e0;
	padding: 4px 8px;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick label:hover {
		color: #72aee6;
	}
}
body.litespeed-darkmode .litespeed-tick label:hover {
	color: #72aee6;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick input[type='checkbox'] {
		background-color: #2d2d30;
		border: 2px solid #5a5a5d;
	}
}
body.litespeed-darkmode .litespeed-tick input[type='checkbox'] {
	background-color: #2d2d30;
	border: 2px solid #5a5a5d;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-tick input[type='checkbox']:checked {
		background-color: #0073aa;
		border-color: #0073aa;
	}
}
body.litespeed-darkmode .litespeed-tick input[type='checkbox']:checked {
	background-color: #0073aa;
	border-color: #0073aa;
}

/* Card headers */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-card-header {
		background-color: #404043;
		border-bottom: 1px solid #5a5a5d;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-card-header {
	background-color: #404043;
	border-bottom: 1px solid #5a5a5d;
	color: #e0e0e0;
}

/* Card action buttons */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-card-action .button-secondary:hover {
		background-color: #484851;
		border-color: #666;
		color: #e0e0e0;
	}
}
body.litespeed-darkmode .litespeed-card-action .button-secondary:hover {
	background-color: #484851;
	border-color: #666;
	color: #e0e0e0;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-card-action .button-primary:hover {
		background-color: #005a87;
		border-color: #004a73;
		color: #ffffff;
	}
}
body.litespeed-darkmode .litespeed-card-action .button-primary:hover {
	background-color: #005a87;
	border-color: #004a73;
	color: #ffffff;
}

@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-h3,
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-panel-para {
		color: revert;
	}
}
body.litespeed-darkmode .litespeed-h3,
body.litespeed-darkmode .litespeed-panel-para {
	color: revert;
}

/* Text gradient for QUIC.cloud branding */
@media (prefers-color-scheme: dark) {
	body[class*="litespeed-cache_page_litespeed"]:not(.litespeed-lightmode) .litespeed-qc-text-gradient {
		background: -webkit-linear-gradient(130deg, #ff69b4, #4db3e6 60%, #a5e7ff);
		-webkit-background-clip: text;
		-webkit-text-fill-color: transparent;
	}
}
body.litespeed-darkmode .litespeed-qc-text-gradient {
	background: -webkit-linear-gradient(130deg, #ff69b4, #4db3e6 60%, #a5e7ff);
	-webkit-background-clip: text;
	-webkit-text-fill-color: transparent;
}
